/* This file is part of Legendes II */
/* Copyright (c) 1998 by
 *   Philippe Clavel (philippe.clavel@supelec.fr)
 *   David Madore (david.madore@ens.fr) */

/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
 * the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* File icons.c */
/* This file contains the routines that read the icon file. */

#include <stdio.h>

#include "lgd0.h"
#include "lgdlib.h"
#include "icons.h"

#define ICON_FILE "icon"

lgd_filedesc_t iconf; /* The icon file itself */

typedef struct color_s {
  unsigned int r,g,b,a; /* Range 0-255 (even though unsigned int) */
} color_t;

typedef color_t icon_t[ICON_SIZE][ICON_SIZE];

#define ICON_CACHE_SIZE 64

struct icon_cache_entry_s {
  u_int32_t n; /* Warning: icon number 0 MUST be all black */
  icon_t ic;
} icon_cache[ICON_CACHE_SIZE];

/* The last icon cache entry examined.  This is used to speed things
 * up a little. */
int last_cache_entry_read;

static int
cache_icon(u_int32_t n)
     /* Make sure an icon is in the icon cache, loading it from disk
      * if necessary.  Return its entry number in the cache. */
{
  int i;
  u_int32_t tab[ICON_SIZE][ICON_SIZE];
  if (icon_cache[last_cache_entry_read].n==n)
    return last_cache_entry_read;
  for (i=0;i<ICON_CACHE_SIZE;i++) {
    if (icon_cache[i].n==n) {
      last_cache_entry_read=i;
      return i;
    }
  }
  i=lgd_random(ICON_CACHE_SIZE);
  icon_cache[i].n=n;
  n=(n&0xffff)+((n>>16)&0xf)+(((n>>20)&0xf)*(((n>>24)&0xf)+1));
  lgd_read_littleendian_int32(1,iconf,(u_int32_t *)tab,
			      n*ICON_SIZE*ICON_SIZE,ICON_SIZE*ICON_SIZE);
  {
    int x,y;
    for (x=0;x<ICON_SIZE;x++) for (y=0;y<ICON_SIZE;y++) {
      icon_cache[i].ic[x][y].r=tab[x][y]&0xff;
      icon_cache[i].ic[x][y].g=(tab[x][y]>>8)&0xff;
      icon_cache[i].ic[x][y].b=(tab[x][y]>>16)&0xff;
      icon_cache[i].ic[x][y].a=(tab[x][y]>>24)&0xff;
    }
  }
  last_cache_entry_read=i;
  return i;
}

void
read_icon(u_int32_t n,int x,int y,
	  unsigned int *r,unsigned int *g,unsigned int *b,unsigned int *a)
     /* Read the icon file to see what icon number n contains at
      * position (x,y). */
{
  *r=icon_cache[cache_icon(n)].ic[x][y].r;
  *g=icon_cache[cache_icon(n)].ic[x][y].g;
  *b=icon_cache[cache_icon(n)].ic[x][y].b;
  *a=icon_cache[cache_icon(n)].ic[x][y].a;
}

void
init_35_icons(void)
{
  lgd_open(1,&iconf,ICON_FILE_PATH ICON_FILE,OPEN_READONLY);
}
