/* This file is part of Legendes II */
/* Copyright (c) 1998 by
 *   Philippe Clavel (philippe.clavel@supelec.fr)
 *   David Madore (david.madore@ens.fr) */

/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
 * the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* File pnm2lgi.c */
/* This file contains a standalone program that will convert a
 * Portable aNyMap to a LeGendes Icon file. */

#include <stdio.h>
#include <string.h>

#include "lgd0.h"
#include "config.h"
#include "lgdlib.h"

#define ICON_SIZE 16

FILE *colorf,*transf;

struct blabla_s {
  unsigned int r,g,b;
} color[ICON_SIZE][ICON_SIZE],trans[ICON_SIZE][ICON_SIZE];

int icon_n;

lgd_filedesc_t inf;

typedef struct color_s {
  unsigned int r,g,b,a; /* Range 0-255 (even though unsigned int) */
} color_t;

typedef color_t icon_t[ICON_SIZE][ICON_SIZE];

icon_t icon;

u_int32_t tab[ICON_SIZE][ICON_SIZE];

void
writepnm(FILE *f,struct blabla_s data[][ICON_SIZE],char format)
{
  int i,j;
  if (f==NULL) {
    return;
  } else {
    if (format==3) fprintf(f,"P3\n");
    else fprintf(f,"P2\n");
    fprintf(f,"# This file was generated by lgi2pnm\n");
    fprintf(f,"%d %d\n",ICON_SIZE,ICON_SIZE);
    fprintf(f,"%d\n",255);
    for (i=0;i<ICON_SIZE;i++) for (j=0;j<ICON_SIZE;j++) {
      if (format==3)
	fprintf(f,"%3d %3d %3d\n",
		data[i][j].r,data[i][j].g,data[i][j].b);
      else
	fprintf(f,"%3d\n",data[i][j].r);
    }
  }
}

int
main(int argc,char **argv)
{
  if (argc<3) {
    fprintf(stderr,"Syntax is %s <input file> <icon number>\n");
    fprintf(stderr,"     [color output file [transparency output file]]\n");
    exit(0);
  }
  lgd_open(1,&inf,argv[1],OPEN_READONLY);
  icon_n=0;
  sscanf(argv[2],"%i",&icon_n);
  if (argc>=4) {
    if (strcmp(argv[3],"-")==0)
      colorf=stdout;
    else if (strcmp(argv[3],".")==0)
      colorf=NULL;
    else if ((colorf=fopen(argv[3],"w"))==NULL) {
      perror("Opening color output file");
      exit(1);
    }
  } else colorf=stdin;
  if (argc>=5) {
    if (strcmp(argv[4],"-")==0)
      transf=stdout;
    else if (strcmp(argv[4],".")==0)
      transf=NULL;
    else if ((transf=fopen(argv[4],"w"))==NULL) {
      perror("Opening transparency output file");
      exit(1);
    }
  } else transf=NULL;
  lgd_read_littleendian_int32(1,inf,(u_int32_t *)tab,
			      icon_n*ICON_SIZE*ICON_SIZE,
			      ICON_SIZE*ICON_SIZE);
  lgd_close(1,inf);
  {
    int i,j;
    for (i=0;i<ICON_SIZE;i++) for (j=0;j<ICON_SIZE;j++) {
      icon[i][j].r=tab[i][j]&0xff;
      icon[i][j].g=(tab[i][j]>>8)&0xff;
      icon[i][j].b=(tab[i][j]>>16)&0xff;
      icon[i][j].a=(tab[i][j]>>24)&0xff;
      trans[j][i].r=icon[i][j].a;
      trans[j][i].g=trans[j][i].r;
      trans[j][i].b=trans[j][i].r;
      color[j][i].r=icon[i][j].r;
      color[j][i].g=icon[i][j].g;
      color[j][i].b=icon[i][j].b;
    }
  }
  writepnm(colorf,color,3);
  writepnm(transf,trans,2);
}
